# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from .tokenexception import TokenException


class InvalidRefreshToken(TokenException):

    def __init__(self):
        super().__init__(
            code='invalid_grant',
            description="The refresh token is malformed."
        )